@@ -76,8 +76,7 @@ entry_changed(GtkWidget *widget, GnomeIconEntry *ientry)
 	char *t = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(ientry->fentry),
 						 FALSE);
 	GdkImlibImage *im;
-	GdkPixmap *pix;
-	GdkBitmap *mask;
+	GtkWidget *pixmap;
 	GtkWidget *child;
 	int w,h;
 
@@ -85,7 +84,7 @@ entry_changed(GtkWidget *widget, GnomeIconEntry *ientry)
 	
 	if(!t || !g_file_test (t,G_FILE_TEST_ISLINK|G_FILE_TEST_ISFILE) ||
 	   !(im = gdk_imlib_load_image (t))) {
-		if(GTK_IS_PIXMAP(child)) {
+		if(GNOME_IS_PIXMAP(child)) {
 			gtk_widget_destroy(child);
 			child = gtk_label_new(_("No Icon"));
 			gtk_widget_show(child);
@@ -107,18 +106,11 @@ entry_changed(GtkWidget *widget, GnomeIconEntry *ientry)
 			h = 48;
 		}
 	}
-	gdk_imlib_render(im,w,h);
-	pix = gdk_imlib_move_image (im);
-	mask = gdk_imlib_move_mask (im);
+	pixmap = gnome_pixmap_new_from_imlib_at_size (im, w, h);
 
-	if(GTK_IS_PIXMAP(child))
-		gtk_pixmap_set(GTK_PIXMAP(child),pix,mask);
-	else {
-		gtk_widget_destroy(child);
-		child = gtk_pixmap_new(pix,mask);
-		gtk_widget_show(child);
-		gtk_container_add(GTK_CONTAINER(ientry->pickbutton),child);
-	}
+	gtk_widget_destroy(child);
+	gtk_widget_show(pixmap);
+	gtk_container_add(GTK_CONTAINER(ientry->pickbutton), pixmap);
 	g_free(t);
 	gdk_imlib_destroy_image(im);
 }
@@ -154,8 +146,6 @@ setup_preview(GtkWidget *widget)
 	GList *l;
 	GtkWidget *pp = NULL;
 	GdkImlibImage *im;
-	GdkPixmap *pix;
-	GdkBitmap *mask;
 	int w,h;
 	GtkWidget *frame = gtk_object_get_data(GTK_OBJECT(widget),"frame");
 	GtkFileSelection *fs = gtk_object_get_data(GTK_OBJECT(frame),"fs");
@@ -186,11 +176,7 @@ setup_preview(GtkWidget *widget)
 			h = 100;
 		}
 	}
-	gdk_imlib_render(im,w,h);
-	pix = gdk_imlib_move_image(im);
-	mask = gdk_imlib_move_mask(im);
-
-	pp = gtk_pixmap_new(pix,mask);
+	pp = gnome_pixmap_new_from_imlib_at_size (im, w, h);
 	gtk_widget_show(pp);
 	gtk_container_add(GTK_CONTAINER(frame),pp);
 
@@ -315,7 +301,17 @@ show_icon_selection(GtkButton * b, GnomeIconEntry * ientry)
 		p = d;
 		if(!g_file_test (p,G_FILE_TEST_ISDIR)) {
 			g_free (p);
-			return;
+			if(fe->default_path)
+				p = g_strdup(fe->default_path);
+			else {
+				/*get around the g_free/free issue*/
+				char *cwd = getcwd(NULL,0);
+				p = g_strdup(cwd);
+				free(cwd);
+			}
+			gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (ientry->fentry))),
+				    p);
+			g_return_if_fail(g_file_test (p,G_FILE_TEST_ISDIR));
 		}
 	}
 	
@@ -609,9 +605,12 @@ gnome_icon_entry_set_icon(GnomeIconEntry *ientry,
 char *
 gnome_icon_entry_get_filename(GnomeIconEntry *ientry)
 {
-	/*this happens if it doesn't exist or isn't an image*/
-	if(!GTK_IS_PIXMAP(GTK_BIN(ientry->pickbutton)->child))
+	GtkWidget *child = GTK_BIN(ientry->pickbutton)->child;
+	
+	/* this happens if it doesn't exist or isn't an image */
+	if(!GNOME_IS_PIXMAP(child))
 		return NULL;
+	
 	return gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(ientry->fentry),
 					      TRUE);
 }
