@@ -133,7 +133,7 @@ receive_fd (int helper_fd)
 
 	if (cmptr == NULL && (cmptr = g_malloc (CONTROLLEN)) == NULL)
 		return -1;
-	msg.msg_control = cmptr;
+	msg.msg_control = (caddr_t) cmptr;
 	msg.msg_controllen = CONTROLLEN;
 
 	if (recvmsg (helper_fd, &msg, 0) <= 0)
@@ -295,6 +295,22 @@ zvt_init_subshell (struct vt_em *vt, char *pty_name, int log)
 	{
 		close (master_pty);
 		login_tty (slave_pty);
+
+		/*
+		 * Reset the child signal handlers
+		 */
+		signal (SIGINT,  SIG_DFL);
+		signal (SIGQUIT, SIG_DFL);
+		signal (SIGCHLD, SIG_DFL);
+
+		/*
+		 * These should be turned off.  Login does turn them off
+		 * If the user shells supports these, they will be turned
+		 * back on
+		 */
+		signal (SIGTSTP, SIG_IGN);
+		signal (SIGTTIN, SIG_IGN);
+		signal (SIGTTOU, SIG_IGN);
 	} else {
 		close (slave_pty);
 
@@ -319,6 +335,7 @@ zvt_init_subshell (struct vt_em *vt, char *pty_name, int log)
 		}
 		
 		vt->keyfd = vt->childfd = master_pty;
+
 	}
 	
 	return vt->childpid;
