@@ -165,11 +165,11 @@ static void vt_set_screen(struct vt_em *vt, int screen)
       }
     });
 
-    if (screen)
-      vt_clear_lines(vt, 0, vt->height);
-
     vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
-    vt->mode = (vt->mode&~VTMODE_ALTSCREEN) | (screen?VTMODE_ALTSCREEN:0);
+    if (screen)
+      vt->mode |= VTMODE_ALTSCREEN;
+    else
+      vt->mode &= ~VTMODE_ALTSCREEN;
   }
 }  
 
@@ -263,12 +263,12 @@ vt_scroll_up(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
-  int blank;
+  uint32 blank;
 
   d(printf("vt_scroll_up count=%d top=%d bottom=%d\n", 
 	   count, vt->scrolltop, vt->scrollbottom));
 
-  blank = vt->attr;
+  blank = vt->attr & VTATTR_CLEARMASK;
 
   while (count>0) {
     /* first, find the line to remove */
@@ -285,7 +285,7 @@ vt_scroll_up(struct vt_em *vt, int count)
     }
 
     for (i=0;i<wn->width;i++)
-      wn->data[i] = vt->attr;
+      wn->data[i] = blank;
 
     if (wn->line == -1) {
       wn->modcount = wn->width;	/* make sure a wrap-scrolled line isn't marked clean */
@@ -318,7 +318,8 @@ vt_scroll_down(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
-
+  uint32 blank = vt->attr & VTATTR_CLEARMASK;
+  
   d(printf("vt_scroll_down count=%d top=%d bottom=%d\n",
 	   count, vt->scrolltop, vt->scrollbottom));
 
@@ -332,7 +333,7 @@ vt_scroll_down(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr;
+      wn->data[i] = blank;
     }
     wn->modcount=0;
     wn->line = -1;		/* flag new line */
@@ -361,6 +362,8 @@ vt_insert_chars(struct vt_em *vt, int count)
   d(printf("vt_insert_chars(%d)\n", count));
   l=vt->this_line;
 
+  count=MIN(count,(vt->width-vt->cursorx));
+
   /* scroll data over count bytes */
   j = (l->width-count)-vt->cursorx;
   for (i=l->width-1;j>0;i--,j--) {
@@ -369,7 +372,7 @@ vt_insert_chars(struct vt_em *vt, int count)
 
   /* clear the rest of the line */
   for (i=vt->cursorx;i<vt->cursorx+count;i++) {
-    l->data[i] = vt->attr;
+    l->data[i] = vt->attr & VTATTR_CLEARMASK;
   }
   l->modcount+=count;
 }
@@ -379,10 +382,14 @@ vt_delete_chars(struct vt_em *vt, int count)
 {
   int i, j;
   struct vt_line *l;
+  uint32 blank;
 
   d(printf("vt_delete_chars(%d)\n", count));
   l=vt->this_line;
 
+  /* check input value for validity */
+  count=MIN(count,(vt->width-vt->cursorx));
+
   /* scroll data over count bytes */
   j = (l->width-count)-vt->cursorx;
   for (i=vt->cursorx;j>0;i++,j--) {
@@ -390,17 +397,18 @@ vt_delete_chars(struct vt_em *vt, int count)
   }
 
   /* clear the rest of the line */
+  blank = l->data[l->width-1] & VTATTR_CLEARMASK & VTATTR_MASK;
   for (i=l->width-count;i<l->width;i++) {
-    l->data[i] = vt->attr;
+    l->data[i] = blank;
   }
   l->modcount+=count;
-
 }
 
 void vt_insert_lines(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
+  uint32 blank = vt->attr & VTATTR_CLEARMASK;
 
   d(printf("vt_insert_lines(%d) (top = %d bottom = %d cursory = %d)\n",
 	   count, vt->scrolltop, vt->scrollbottom, vt->cursory));
@@ -414,7 +422,7 @@ void vt_insert_lines(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr;
+      wn->data[i] = blank;
     }
     wn->modcount=0;		/* set as 'unchanged' so the scroll
 				   routine can update it.
@@ -438,6 +446,7 @@ void vt_delete_lines(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
+  uint32 blank = vt->attr & VTATTR_CLEARMASK;
 
   d(printf("vt_delete_lines(%d)\n", count));
   /* FIXME: do this properly */
@@ -449,7 +458,7 @@ void vt_delete_lines(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr;
+      wn->data[i] = blank;
     }
     wn->modcount=0;
     /*wn->line=vt->scrollbottom;*/
@@ -468,6 +477,7 @@ void vt_clear_lines(struct vt_em *vt, int top, int count)
 {
   struct vt_line *wn, *nn;
   int i;
+  uint32 blank=vt->attr&VTATTR_CLEARMASK;
 
   d(printf("vt_clear_lines(%d, %d)\n", top, count));
   i=1;
@@ -475,7 +485,7 @@ void vt_clear_lines(struct vt_em *vt, int top, int count)
   nn=wn->next;
   while(nn && count>=0) {
     for(i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr;
+      wn->data[i] = blank;
     }
     wn->modcount = wn->width;
     count--;
@@ -488,12 +498,16 @@ void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col)
 {
   struct vt_line *this_line;
   int i;
+  uint32 blank = vt->attr&VTATTR_CLEARMASK;
 
   d(printf("vt_clear_line_portion()\n"));
 
+  start_col = MIN(start_col, vt->width);
+  end_col = MIN(end_col, vt->width);
+
   this_line = vt->this_line;
   for(i=start_col;i<end_col;i++) {
-    this_line->data[i] = vt->attr;
+    this_line->data[i] = blank;
   }
   this_line->modcount+=(this_line->width-vt->cursorx);
 }
@@ -551,6 +565,13 @@ static void vt_tab(struct vt_em *vt)
   d(printf("tab\n"));
 
   l = vt->this_line;
+  if (vt->cursorx>=vt->width) {
+      if (!(vt->mode & VTMODE_WRAPOFF)) {
+	  vt->cursorx = 0;
+	  vt_lf(vt);			/* goto next line */
+      } else
+	  return;
+  }
   c = l->data[vt->cursorx] & VTATTR_DATAMASK;
 
   /* dont store tab over a space - will affect attributes */
@@ -563,9 +584,13 @@ static void vt_tab(struct vt_em *vt)
 
   /* move cursor to new tab position */
   vt->cursorx = (vt->cursorx + 8) & (~7);
-  if (vt->cursorx >= vt->width) {
-    vt->cursorx = 0;
-    vt_lf(vt);			/* goto next line */
+  if (vt->cursorx > vt->width) {
+      if (vt->mode & VTMODE_WRAPOFF) {
+	  vt->cursorx=vt->width-1;
+      } else {
+	  vt->cursorx = 0;
+	  vt_lf(vt);			/* goto next line */
+      }
   }
 }
 
@@ -713,13 +738,13 @@ vt_restore_cursor(struct vt_em *vt)
   d(printf("restore cursor\n"));
   vt->cursorx = vt->savex;
   vt->cursory = vt->savey;
-  vt->mode = (vt->savemode & (VTMODE_INSERT | VTMODE_WRAPOFF | VTMODE_APP_CURSOR | VTMODE_RELATIVE | VTMODE_ALTSCREEN));
+  vt->mode = (vt->savemode & (VTMODE_INSERT | VTMODE_WRAPOFF | VTMODE_APP_CURSOR | VTMODE_RELATIVE));
   vt->attr = vt->saveattr;
   vt->remaptable = vt->saveremaptable;
 
   /* incase the save/restore was across re-sizes */
-  if (vt->cursorx >= vt->width)
-    vt->cursorx = vt->width-1;
+  if (vt->cursorx > vt->width)
+    vt->cursorx = vt->width;
   if (vt->cursory >= vt->height)
     vt->cursory = vt->height-1;
 
@@ -1283,6 +1308,7 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	  /* d(printf("literal %c\n", c)); */
 	  vt->cursorx++;
 	} else if (mode & VT_CON) {
+	  vt->argcnt=0;
 	  process(vt);
 	} else if (c==27) {
 	  state=1;
@@ -1319,11 +1345,13 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 
 
       case 2:
+	d(printf("state 2: %d '%c'\n",vt->argcnt, c));
 	if (mode & VT_ARG) {
-	  if (vt->outptr<vt->outend) /* truncate excessive args */
+	  if (vt->argptr < &vt->args[VTPARAM_MAXARGS]
+	      && vt->outptr<vt->outend) /* truncate excessive args */
 	    *(vt->outptr)++=c;
 	} else if (c==';' || c==':') {	/* looking for 'arg;arg;...' */
-	  if (vt->argcnt<VTPARAM_MAXARGS) { /* goto next argument */
+	  if (vt->argptr < &vt->args[VTPARAM_MAXARGS]) { /* goto next argument */
 	    *(vt->outptr)=0;
 	    vt->argptr++;
 	    vt->outptr = vt->argptr[0];
@@ -1338,6 +1366,8 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	  process(vt);
 	  state=0;
 	} else if (mode & VT_CON) {
+	  d(printf("control'%c'\n", c));
+	  vt->argcnt=0;
 	  process(vt);
 	} else {
 	  d(printf("unknown option '%c'\n", c));
@@ -1349,6 +1379,7 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	/* \EOx */
       case 3:
 	if (mode & VT_EXO) {
+	  vt->argcnt=0;
 	  process(vt);
 	}	/* ignore otherwise */
 	state=0;
@@ -1375,6 +1406,7 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	/* \E?x */
       case 5:
 	vt->args[0][1]=c;
+	vt->argcnt=0;
 	modes[vt->args[0][0]].process(vt);
 	state=0;
 	break;
@@ -1405,10 +1437,9 @@ struct vt_line *vt_newline(struct vt_em *vt)
   l->line = 0;
   l->modcount = vt->width;
 
-  for (i = 0; i < vt->width; i++)
-    {
-      l->data[i] = vt->attr;
-    }
+  for (i = 0; i < vt->width; i++) {
+    l->data[i] = vt->attr & VTATTR_CLEARMASK;
+  }
 
   return l;
 }
@@ -1722,7 +1753,7 @@ vt_resize_lines(struct vt_line *wn, int width, uint32 default_attr)
  */
 void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixheight)
 {
-  int i, count, pass, old_width, remove_bottom;
+  int i, count, pass, old_width;
   uint32 c;
   struct vt_line *wn, *nn;
 
@@ -1740,7 +1771,6 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
   if (height < vt->height) {
 
     count = vt->height - height;
-    remove_bottom = 0;
 
     d(printf("removing %d lines to smaller window\n", count));
 
@@ -1750,13 +1780,7 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
        * switch to a mode where we just dispose of the lines at the bottom
        * of the terminal
        */
-      if ((struct vt_line *)vt->lines.head == vt->this_line ||
-	  (struct vt_line *)vt->lines_alt.head == vt->this_line)
-	{
-	  remove_bottom = 1;
-	}
-
-      if (remove_bottom)
+      if (vt->cursory==0)
 	{
 	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines)) ) {
 	    g_free(wn);
@@ -1789,6 +1813,8 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
 	  /* repeat for backbuffer */
 	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_back)) )
 	    g_free(wn);
+
+	  vt->cursory--;
 	}
 
       count--;
@@ -1846,6 +1872,11 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
 	vt_list_addhead(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
 
 	vt->scrollbacklines--;	/* since we just nuked one */
+
+	if ((-vt->scrollbackoffset)>vt->scrollbacklines){
+	  vt->scrollbackoffset++;
+	}
+
 	vt->cursory++;		/* fix up cursor */
       } else {
 	d(printf("adding line to bottom of screen\n"));
@@ -1868,9 +1899,9 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
   /* now, scan all lines visible, and make them the right width
    * for all 3 'buffers', onscreen, offscreen and alternate
    */
-  vt_resize_lines((struct vt_line *) vt->lines.head, width, vt->attr);
-  vt_resize_lines((struct vt_line *) vt->lines_back.head, width, vt->attr);
-  vt_resize_lines((struct vt_line *) vt->lines_alt.head, width, vt->attr);
+  vt_resize_lines((struct vt_line *) vt->lines.head, width, vt->attr & VTATTR_CLEARMASK);
+  vt_resize_lines((struct vt_line *) vt->lines_back.head, width, vt->attr & VTATTR_CLEARMASK);
+  vt_resize_lines((struct vt_line *) vt->lines_alt.head, width, vt->attr & VTATTR_CLEARMASK);
 
   /* re-fix 'this line' pointer */
   vt->this_line = (struct vt_line *) vt_list_index(&vt->lines, vt->cursory);
