@@ -860,7 +860,7 @@ zvt_term_size_allocate (GtkWidget     *widget,
 			      allocation->width,
 			      allocation->height);
 
-      term_width = allocation->width - (2 * widget->style->klass->xthickness) - PADDING;
+      term_width = allocation->width - (2 * widget->style->klass->xthickness);
       term_height = allocation->height - (2 * widget->style->klass->ythickness);
 
       gdk_window_move_resize (term->term_window,
@@ -1263,10 +1263,10 @@ zvt_term_button_press (GtkWidget      *widget,
       vx->selectiontype = VT_SELTYPE_CHAR|VT_SELTYPE_BYSTART;
       break;
     case GDK_2BUTTON_PRESS:
-      vx->selectiontype = VT_SELTYPE_WORD|VT_SELTYPE_BYSTART;
+      vx->selectiontype = VT_SELTYPE_WORD|VT_SELTYPE_BYSTART|VT_SELTYPE_MOVED;
       break;
     case GDK_3BUTTON_PRESS:
-      vx->selectiontype = VT_SELTYPE_LINE|VT_SELTYPE_BYSTART;
+      vx->selectiontype = VT_SELTYPE_LINE|VT_SELTYPE_BYSTART|VT_SELTYPE_MOVED;
       break;
     default:
       break;
@@ -1279,17 +1279,18 @@ zvt_term_button_press (GtkWidget      *widget,
     vx->selendy = y;
     
     /* reset 'drawn' screen (to avoid mis-refreshes) */
-    if (!vx->selected)
-      {
-	vx->selstartxold = x;
-	vx->selstartyold = y;
-	vx->selendxold = x;
-	vx->selendyold = y;
-	vx->selected =1;
-      }
-    
-    vt_fix_selection(vx);
-    vt_draw_selection(vx);	/* handles by line/by word update */
+    if (!vx->selected) {
+      vx->selstartxold = x;
+      vx->selstartyold = y;
+      vx->selendxold = x;
+      vx->selendyold = y;
+      vx->selected =1;
+    }
+
+    if (event->type != GDK_BUTTON_PRESS) {
+      vt_fix_selection(vx);
+      vt_draw_selection(vx);	/* handles by line/by word update */
+    }
 
     d( printf("selection starting %d %d\n", x, y) );
 
@@ -1408,17 +1409,20 @@ zvt_term_button_release (GtkWidget      *widget,
 
     gtk_grab_remove (widget);
     gdk_pointer_ungrab (0);
-    
-    vt_fix_selection(vx);
-    vt_draw_selection(vx);
+
+    if (vx->selectiontype & VT_SELTYPE_MOVED) {
+      vt_fix_selection(vx);
+      vt_draw_selection(vx);
+          
+      vt_get_selection(vx, 0);
+      
+      gtk_selection_owner_set (widget,
+			       GDK_SELECTION_PRIMARY,
+			       event->time);
+    }
 
     vx->selectiontype = VT_SELTYPE_NONE; /* 'turn off' selecting */
-    
-    vt_get_selection(vx, 0);
 
-    gtk_selection_owner_set (widget,
-			     GDK_SELECTION_PRIMARY,
-			     event->time);
   }
 
   return FALSE;
@@ -1457,6 +1461,8 @@ zvt_term_motion_notify (GtkWidget      *widget,
       vx->selstartx = x;
       vx->selstarty = y + vx->vt.scrollbackoffset;
     }
+
+    vx->selectiontype |= VT_SELTYPE_MOVED;
     
     vt_fix_selection(vx);
     vt_draw_selection(vx);
@@ -1554,6 +1560,12 @@ zvt_term_selection_received (GtkWidget *widget, GtkSelectionData *selection_data
   /* paste selection into window! */
   if (selection_data->length)
     {
+      int i;
+      char *ctmp = selection_data->data;
+
+      for(i = 0; i < selection_data->length; i++)
+	if(ctmp[i] == '\n') ctmp[i] = '\r';
+
       if (term->scroll_on_keystroke)
 	zvt_term_scroll (term, 0);
       vt_writechild(&vx->vt, selection_data->data, selection_data->length);
